<?php
namespace FiloBlu\ProductUrlTools\Console\Command;

use Symfony\Component\Console\Command\Command;
use Symfony\Component\Console\Input\InputOption;
use Symfony\Component\Console\Input\InputArgument;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\OutputInterface;
use Magento\UrlRewrite\Service\V1\Data\UrlRewrite;
use Magento\UrlRewrite\Model\UrlPersistInterface;
use Magento\CatalogUrlRewrite\Model\ProductUrlRewriteGenerator;
use Magento\Catalog\Model\ResourceModel\Product\Collection;
use Magento\Store\Model\Store;
use Magento\Framework\App\State;

class RegenerateProductUrlKeyCommand extends Command
{
    /**
     * @var ProductUrlRewriteGenerator
     */
    protected $productUrlRewriteGenerator;

    /**
     * @var UrlPersistInterface
     */
    protected $urlPersist;

    /**
     * @var ProductRepositoryInterface
     */
    protected $collection;

    /**
     * @var \Magento\Framework\App\State
     */
    protected $state;
    protected $logger;
    protected $action;


    public function __construct(
        State $state,
        Collection $collection,
        ProductUrlRewriteGenerator $productUrlRewriteGenerator,
        UrlPersistInterface $urlPersist,
        \Psr\Log\LoggerInterface $logger,
        \Magento\Catalog\Model\Product\Attribute\Source\Status $productStatus,
        \Magento\Catalog\Model\Product\Visibility $productVisibility,
        \Magento\Catalog\Model\Product\Action $action

    ) {
        $this->state = $state;
        $this->collection = $collection;
        $this->productUrlRewriteGenerator = $productUrlRewriteGenerator;
        $this->urlPersist = $urlPersist;
        $this->logger = $logger;
        $this->productVisibility = $productVisibility;
        $this->productStatus = $productStatus;
        $this->action= $action;

        parent::__construct();
    }

    protected function configure()
    {
        $this->setName('tools:regenerateurlkey')
            ->setDescription('Regenerate urlkey for given products')
            ->addArgument(
                'pids',
                InputArgument::IS_ARRAY,
                'Products to regenerate'
            )
            ->addOption(
                'store', 's',
                InputOption::VALUE_REQUIRED,
                'Use the specific Store View',
                Store::DEFAULT_STORE_ID
            )
            ->addOption(
                'all_visible', 'a',
                InputOption::VALUE_OPTIONAL,
                'Use for all product visible'
            )
        ;
        return parent::configure();
    }

    public function execute(InputInterface $inp, OutputInterface $out)
    {

        $store_id = $inp->getOption('store');
        $all_visible=$inp->getOption('all_visible');

        $this->collection->addStoreFilter($store_id)->setStoreId($store_id);
        $pids = $inp->getArgument('pids');

        if( !empty($pids) )
            $this->collection->addIdFilter($pids);

        $this->collection->addAttributeToSelect(['name', 'url_key']);
        if($all_visible)
        {
            $this->all_productVisible();
        }
        $list = $this->collection->load();


        foreach($list as $product)
        {

            if($store_id === Store::DEFAULT_STORE_ID)
                $product->setStoreId($store_id);

            $store_url_key = ($store_id) ? $store_id : 0;

            if (!$product->getName()) {
                $this->logger->info('name is empty for product : '. $product->getId() . ' in store view ' . $store_id);
                continue;
            }

            $urlKey = $product->formatUrlKey($product->getName() .'-'.$store_url_key. $product->getId());

            $pids = array($product->getId());
            $attributesData = array("url_key" => $urlKey);

            try {

                $this->action->updateAttributes($pids, $attributesData, $store_id);
                $this->logger->info('processed urlkey for product with id : '. $product->getId());
            }
            catch(\Exception $e) {
                $out->writeln('<error>Duplicated url for '. $product->getId() .' '. $e->getMessage(). ' </error>');
            }
        }
    }

    private function all_productVisible()
    {
        $this->collection->addAttributeToFilter('status', ['in' => $this->productStatus->getVisibleStatusIds()]);
        $this->collection->setVisibility($this->productVisibility->getVisibleInSiteIds());
    }
}

